@@ -1,3 +1,5 @@
+require 'puppet/concurrent/thread_local_singleton'
+
 module Puppet::Pops
 module Time
   NSECS_PER_USEC = 1000
@@ -259,7 +261,7 @@ module Time
           add_unless_zero(result, KEY_NANOSECONDS, n % 1000)
           n /= 1000
           add_unless_zero(result, KEY_MICROSECONDS, n % 1000)
-          add_unless_zero(result, KEY_MILLISECONDS, n /= 1000)
+          add_unless_zero(result, KEY_MILLISECONDS, n / 1000)
         end
         result[KEY_NEGATIVE] = true if negative?
       end
@@ -581,9 +583,7 @@ module Time
 
     # Parses a string into a Timestamp::Format instance
     class FormatParser
-      def self.singleton
-        @singleton
-      end
+      extend Puppet::Concurrent::ThreadLocalSingleton
 
       def initialize
         @formats = Hash.new { |hash, str| hash[str] = internal_parse(str) }
@@ -706,8 +706,6 @@ module Time
         end
         Format.new(str, bld)
       end
-
-      @singleton = FormatParser.new
     end
 
     class Format
